import numpy as np
import math
import matplotlib.pyplot as plt

def NB_GenerateData(D, p, count):
    '''
    This function generates data to follow the probabilities that the Naive Bayes model learned
    D is the matrix of P(X|Y) probabilities
    p is the P(Y) prior probabilities
    count is the number of data points we will generate
    '''
    xData = np.zeros((count, D.shape[1]))
    countTrue = np.sum(np.random.binomial(1, p, count))
    countFalse = count-countTrue
    trueY = np.ones(countTrue)+1
    falseY = np.ones(countFalse)
    yData = np.concatenate((trueY, falseY))
    for f in range(D.shape[1]):
        trueX = np.random.binomial(1,D[1][f],countTrue)
        falseX = np.random.binomial(1,D[0][f],countFalse)
        tempX = np.concatenate((trueX, falseX))
        xData[:,f] = tempX
    return xData, yData

def augmentFeatures(XTrain, yTrain, XTest, yTest):
    te_vals = {}
    teaug_vals = {}

    np.random.seed(0)
    for i in range(150,451,30):
        te_vals[i] = 0
        teaug_vals[i] = 0

    for i in range(10):
        fset = np.random.choice(XTrain.shape[1],50,replace=False)

        for m in range(150,451,30):
            D = NB_XGivenY(XTrain[0:m], yTrain[0:m])
            p = NB_YPrior(yTrain[0:m])

            yHatTest = NB_Classify(D, p, XTest)

            testAcc = NB_ClassificationAccuracy(yHatTest, yTest)

            te_vals[m] += testAcc
            
            #augmebting features
            xtrain_aug = np.repeat(XTrain[0:m,fset],100,axis=1)
            xtest_aug = np.repeat(XTest[:,fset],100,axis=1)
            xTrain = np.hstack([XTrain[0:m],xtrain_aug])
            xTest = np.hstack([XTest,xtest_aug])
            D = NB_XGivenY(xTrain, yTrain[0:m])
            p = NB_YPrior(yTrain[0:m])

            yHatTest = NB_Classify(D, p, xTest)

            testAcc = NB_ClassificationAccuracy(yHatTest, yTest)
            teaug_vals[m] += testAcc

            

    for i in range(150,451,30):
        te_vals[i] /= 10
        teaug_vals[i] /= 10

    plt.plot(list(te_vals.keys()),list(te_vals.values()),label='Original dataset')
    plt.plot(list(teaug_vals.keys()),list(teaug_vals.values()),label='With augmented features')
    plt.legend()
    plt.xlabel('m')
    plt.ylabel('Test Accuracy')
    plt.savefig('./q9.pdf')


def NB_XGivenY(XTrain, yTrain, a=0.001, b=0.9):

    THRESHOLD = 1e-5

    # TODO: Implement P(X|Y) with Beta(a,1+b) Prior


def NB_YPrior(yTrain):

    # TODO: Implement P(Y)


def NB_Classify(D, p, X):

    # TODO: Implement Label Predition Vector of X


def NB_ClassificationAccuracy(yHat, yTruth):

    # TODO: Compute the Classificaion Accuracy of yHat Against yTruth


if __name__ == "__main__":

    import pickle
    with open("hw1data.pkl", "rb") as f:
        data = pickle.load(f)

    XTrain = data["XTrain"]
    yTrain = data["yTrain"]

    XTest = data["XTest"]
    yTest = data["yTest"]

    Vocab = data["Vocabulary"]